'use strict';

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _payment = require('payment');

var _payment2 = _interopRequireDefault(_payment);

require('./styles.scss');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class ReactCreditCards extends _react2.default.Component {
  constructor(props) {
    super(props);

    this.state = {
      type: {
        name: 'unknown',
        maxLength: 16
      }
    };
  }

  componentWillMount() {
    this.setCards();
  }

  componentDidMount() {
    const { number } = this.props;
    this.updateType(number);
  }

  componentWillReceiveProps(nextProps) {
    const { acceptedCards, number } = this.props;

    const {
      acceptedCards: nextAcceptedCards,
      number: nextNumber
    } = nextProps;

    if (number !== nextNumber) {
      this.updateType(nextNumber);
    }

    if (acceptedCards.toString() !== nextAcceptedCards.toString()) {
      this.setCards(nextProps);
    }
  }

  setCards(props = this.props) {
    const { acceptedCards } = props;
    let newCardArray = [];

    if (acceptedCards.length) {
      _payment2.default.getCardArray().forEach(d => {
        if (acceptedCards.includes(d.type)) {
          newCardArray.push(d);
        }
      });
    } else {
      newCardArray = newCardArray.concat(_payment2.default.getCardArray());
    }

    _payment2.default.setCardArray(newCardArray);
  }

  updateType(number) {
    const { callback } = this.props;
    const type = _payment2.default.fns.cardType(number) || 'unknown';

    let maxLength = 16;

    if (type === 'amex') {
      maxLength = 15;
    } else if (type === 'dinersclub') {
      maxLength = 14;
    } else if (['hipercard', 'mastercard', 'visa'].includes(type)) {
      maxLength = 19;
    }

    const typeState = {
      issuer: type,
      maxLength
    };
    const isValid = _payment2.default.fns.validateCardNumber(number);

    this.setState({
      type: typeState
    });

    /* istanbul ignore else */
    if (typeof callback === 'function') {
      callback(typeState, isValid);
    }
  }

  formatNumber() {
    const { type } = this.state;
    const { number } = this.props;

    let { maxLength } = type;
    let string = typeof number === 'number' ? number.toString() : number;
    if (isNaN(parseInt(number, 10))) {
      string = '';
    }

    if (type.maxLength > 16) {
      maxLength = string.length <= 16 ? 16 : type.maxLength;
    }

    if (string.length > maxLength) {
      string = string.slice(0, maxLength);
    }

    while (string.length < maxLength) {
      string += '•';
    }

    if (['amex', 'dinersclub'].includes(type.issuer)) {
      const format = [0, 4, 10];
      const limit = [4, 6, 5];
      string = `${string.substr(format[0], limit[0])} ${string.substr(format[1], limit[1])} ${string.substr(format[2], limit[2])}`;
    } else if (number.length > 16) {
      const format = [0, 4, 8, 12];
      const limit = [4, 7];
      string = `${string.substr(format[0], limit[0])} ${string.substr(format[1], limit[0])} ${string.substr(format[2], limit[0])} ${string.substr(format[3], limit[1])}`;
    } else {
      for (let i = 1; i < maxLength / 4; i++) {
        const space_index = i * 4 + (i - 1);
        string = `${string.slice(0, space_index)} ${string.slice(space_index)}`;
      }
    }

    return string;
  }

  formatExpiry() {
    const { expiry = '' } = this.props;

    const value = expiry.toString();
    const maxLength = 6;
    let string = value || '••/••';

    if (value.match(/\//)) {
      string = expiry.replace('/', '');
    }

    if (!string.match(/^[0-9]*$/)) {
      return '••/••';
    }

    while (string.length < 4) {
      string += '•';
    }

    return `${string.slice(0, 2)}/${string.slice(2, maxLength)}`;
  }

  render() {
    const { type } = this.state;
    const { cvc, focused, locale, name, placeholders } = this.props;
    const number = this.formatNumber();
    const expiry = this.formatExpiry();

    return _react2.default.createElement(
      'div',
      { key: 'Cards', className: 'rccs' },
      _react2.default.createElement(
        'div',
        {
          className: ['rccs__card', `rccs__card--${type.issuer}`, focused === 'cvc' && type.issuer !== 'amex' ? 'rccs__card--flipped' : ''].join(' ').trim()
        },
        _react2.default.createElement(
          'div',
          { className: 'rccs__card--front' },
          _react2.default.createElement('div', { className: 'rccs__card__background' }),
          _react2.default.createElement('div', { className: 'rccs__issuer' }),
          _react2.default.createElement(
            'div',
            {
              className: ['rccs__cvc__front', focused === 'cvc' ? 'rccs--focused' : ''].join(' ').trim()
            },
            cvc
          ),
          _react2.default.createElement(
            'div',
            {
              className: ['rccs__number', number.replace(/ /g, '').length > 16 ? 'rccs__number--large' : '', focused === 'number' ? 'rccs--focused' : '', number.substr(0, 1) !== '•' ? 'rccs--filled' : ''].join(' ').trim()
            },
            number
          ),
          _react2.default.createElement(
            'div',
            {
              className: ['rccs__name', focused === 'name' ? 'rccs--focused' : '', name ? 'rccs--filled' : ''].join(' ').trim()
            },
            name || placeholders.name
          ),
          _react2.default.createElement(
            'div',
            {
              className: ['rccs__expiry', focused === 'expiry' ? 'rccs--focused' : '', expiry.substr(0, 1) !== '•' ? 'rccs--filled' : ''].join(' ').trim()
            },
            _react2.default.createElement(
              'div',
              { className: 'rccs__expiry__valid' },
              locale.valid
            ),
            _react2.default.createElement(
              'div',
              { className: 'rccs__expiry__value' },
              expiry
            )
          ),
          _react2.default.createElement('div', { className: 'rccs__chip' })
        ),
        _react2.default.createElement(
          'div',
          { className: 'rccs__card--back' },
          _react2.default.createElement('div', { className: 'rccs__card__background' }),
          _react2.default.createElement('div', { className: 'rccs__stripe' }),
          _react2.default.createElement('div', { className: 'rccs__signature' }),
          _react2.default.createElement(
            'div',
            {
              className: ['rccs__cvc', focused === 'cvc' ? 'rccs--focused' : ''].join(' ').trim()
            },
            cvc
          ),
          _react2.default.createElement('div', { className: 'rccs__issuer' })
        )
      )
    );
  }
}

ReactCreditCards.propTypes = {
  acceptedCards: _propTypes2.default.array,
  callback: _propTypes2.default.func,
  cvc: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number]).isRequired,
  expiry: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number]).isRequired,
  focused: _propTypes2.default.string,
  locale: _propTypes2.default.shape({
    valid: _propTypes2.default.string
  }),
  name: _propTypes2.default.string.isRequired,
  number: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number]).isRequired,
  placeholders: _propTypes2.default.shape({
    name: _propTypes2.default.string
  })
};
ReactCreditCards.defaultProps = {
  acceptedCards: [],
  locale: {
    valid: 'valid thru'
  },
  placeholders: {
    name: 'YOUR NAME HERE'
  }
};
exports.default = ReactCreditCards;